require 'minitest/autorun'
require 'osc/reservations'

require 'ostruct'
require 'yaml'

# Stub out command line methods
module Open3
  def popen3(*cmd,&block)
    line = cmd[0]

    wait_thr = nil
    if line =~ /NIL_RSV/
      wait_thr = OpenStruct.new(value: OpenStruct.new(:'success?' => false))
    else
      wait_thr = OpenStruct.new(value: OpenStruct.new(:'success?' => true))
    end

    file = "osc_rsv.xml" if line =~ /mrsvctl/
    file = "osc_rsvs.xml" if line =~ /mrsvctl/ && line =~ /ALL/
    File.open("test/files/#{file}", "r") do |f|
      yield "", f, "", wait_thr
    end
  end
  module_function :popen3
end

# Stub out PBS methods
module PBS
  class Conn
    def self.batch(name)
      name
    end
  end
  class Query
    def initialize(opts)
      @conn = opts[:conn]
      @type = opts[:type]
    end
    def find(opts)
      YAML.load_file("test/files/osc_#{@conn}_#{@type}_#{opts[:id]}.yml")
    end
  end
end

class TestOSCMoab < Minitest::Test
  def setup
    @batch = OpenStruct.new(server: 'a.b.c', torque_name: 'oakley', mrsvctl: 'mrsvctl')
    @adapter = OSC::Reservations::Adapters::OSCMoab.new
  end

  def test_osc_moab
    assert_respond_to @adapter, :query_reservation
    assert_respond_to @adapter, :query_reservations
    assert_respond_to @adapter, :submit_reservation
  end

  def test_query_reservation
    rsv = @adapter.query_reservation(@batch, 'NIL_RSV')
    assert_equal nil, rsv

    rsv = @adapter.query_reservation(@batch, 'wiag.1809')
    test_wiag_rsv(rsv)
  end

  def test_query_reservations
    rsv_list = @adapter.query_reservations(@batch)
    assert_equal 2, rsv_list.length

    rsv = rsv_list.first
    test_wiag_rsv(rsv)

    # reservation that hasn't started yet
    rsv = rsv_list[1]
    assert_equal 4, rsv.nodes.length
    assert_operator Time.now, :<, rsv.starttime
  end

  def test_submit_reservation
    @adapter.submit_reservation(@batch, nil)
  end

  private

  def test_wiag_rsv(rsv)
    assert_equal "wiag.1809", rsv.id
    assert_equal Time.at(1433854480), rsv.starttime
    assert_equal Time.at(1633854480), rsv.endtime
    assert_equal "judithg", rsv.auser
    assert_equal ["bmcmichael", "jnicklas", "awe0011", "ndem0009", "efranz", "ndem0010", "bgohar"], rsv.users

    nodes = rsv.nodes
    assert_equal 4, nodes.length

    # node with no jobs
    node = nodes[0]
    assert_equal "n0613", node.id
    assert_equal 12, node.np
    assert_equal ["cbb15", "partdual", "udapl"], node.props
    assert_equal [], node.jobs
    assert_equal [], node.users

    # node with one job using all cores
    node = nodes[3]
    assert_equal "n0609", node.id
    assert_equal ["4691489.oak-batch.osc.edu"], node.jobs
    assert_equal ["jnicklas"], node.users

    # node with multiple jobs on it
    node2 = nodes[2]
    assert_equal "n0611", node2.id
    assert_equal ["4691490.oak-batch.osc.edu", "4691491.oak-batch.osc.edu"], node2.jobs
    assert_equal ["jnicklas"], node2.users
  end
end
