require 'minitest/autorun'
require 'osc/reservations'

BATCH_CONFIG = {
  'batch1' => {
    'adapter' => {
      'type' => 'FakeAdapter'
    },
    'batch' => {
      'server' => 'batch1.domain.com',
      'extra1' => 'EXTRA1'
    }
  }
}

class FakeAdapter
end

class TestQuery < Minitest::Test
  def test_query
    query = OSC::Reservations::Query.new 'ADAPTER', 'BATCH'
    assert_respond_to query, :reservation
    assert_respond_to query, :reservations
  end

  def test_reservation
    adapter = Minitest::Mock.new
    adapter.expect(:query_reservation, 'RSV', ['BATCH', 123])
    query = OSC::Reservations::Query.new adapter, 'BATCH'
    rsv = query.reservation 123
    assert_equal 'RSV', rsv
    adapter.verify
  end

  def test_reservations
    adapter = Minitest::Mock.new
    adapter.expect(:query_reservations, ['RSV1', 'RSV2'], ['BATCH'])
    query = OSC::Reservations::Query.new adapter, 'BATCH'
    rsv_list = query.reservations
    assert_equal ['RSV1', 'RSV2'], rsv_list
    adapter.verify
  end

  def test_predefined_batch_servers
    OSC::Reservations.stub :batch_config, BATCH_CONFIG do
      assert_respond_to OSC::Reservations::Query, :batch1
      refute_respond_to OSC::Reservations::Query, :batch_unknown

      mock_adapter = Minitest::Mock.new
      mock_adapter.expect(:call, nil)
      mock_batch = Minitest::Mock.new
      mock_batch.expect(:call, nil, ['batch1.domain.com', {'extra1' => 'EXTRA1'}])
      FakeAdapter.stub :new, mock_adapter do
        OSC::Reservations::Batch.stub :new, mock_batch do
          assert_instance_of OSC::Reservations::Query, OSC::Reservations::Query.batch1
        end
      end
      mock_batch.verify
      mock_adapter.verify
    end
  end
end
